package org.netkernel.lang.dpml.ast.impl;


import org.netkernel.container.ILogger;
import org.netkernel.lang.dpml.ast.*;
import org.netkernel.lang.dpml.endpoint.DPMLRuntime;
import org.netkernel.lang.dpml.endpoint.DPMLSubrequestContext;
import org.netkernel.layer0.nkf.INKFRequest;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFRequestReadOnly;
import org.netkernel.layer0.nkf.NKFException;
import org.netkernel.request.IRequestScopeLevel;
import org.netkernel.request.IResponseMeta;
import org.netkernel.request.impl.MetaImpl;
import org.netkernel.request.impl.RequestScopeLevelImpl;
import org.netkernel.urii.ISpace;

import java.util.List;

public class AugmentedClosure extends VariableImpl implements IValueVariable
{
	private String mClosure;
	private String mVerb;
	private List mArguments;
	private ISpace mDPMLContext;	
	
	public AugmentedClosure(String aAssignment, IScope aScope, ILocation aLocation, String aClosure, String aVerb, List aArguments,ISpace aDPMLContext)
	{	super(aAssignment,aScope,aLocation);
		mClosure=aClosure;
		mVerb=aVerb;
		mArguments=aArguments;
		mDPMLContext=aDPMLContext;
	}
	
	public IValue getValue(INKFRequestContext aContext, IRequestScopeLevel aSubRequestContext, Class aRepresentationClass) throws NKFException
	{	
		IValue result;
		
		/*
		if (aContext.shouldLog(INKFRequestContext.LEVEL_INFO))
		{	System.out.print("calling closure "+mClosure+" with arguments of ");
			for (int i=0; i<mArguments.size(); i++)
			{	IArgument arg=(IArgument)mArguments.get(i);
				System.out.print(arg.getName()+"="+arg.getVariable().getName()+" ");
			}			
			System.out.println("");
		}
		*/
		
		IVariable toCall= DPMLRuntime.resolveName(mClosure,getScope(),aSubRequestContext);
		if (toCall==null)
		{	throw aContext.createFormattedException("EX_DPML_RUNTIME","MSG_RUN_NOT_FOUND",null,null,mClosure);
		}
		
		if ( (mArguments.size()>0 || !mVerb.equals("SOURCE")) && (!(toCall instanceof Closure) && !(toCall instanceof Sequence) ))
		{	throw aContext.createFormattedException("EX_DPML_RUNTIME","MSG_RUN_NOT_CLOSURE",null,null,mClosure);
		}

		DPMLSubrequestContext ctx= new DPMLSubrequestContext(aSubRequestContext,mDPMLContext,aContext.getCWU());
		for (int i=0; i<mArguments.size(); i++)
		{	IArgument arg=(IArgument)mArguments.get(i);
		
			IVariable var = DPMLRuntime.processArgument(arg,aContext,aSubRequestContext,false);
			ctx.putArgument(arg.getName(),var);

		}
		IResponseMeta meta = MetaImpl.NEVER_EXPIRED;					
		IVariable var = new LiteralValue(null,null,getLocation(),mVerb,meta);
		ctx.putArgument(IArgument.ARG_VERB,var);
		IRequestScopeLevel rctx=RequestScopeLevelImpl.createDurableScopeLevel(ctx,aSubRequestContext);
		
		if (DPMLRuntime.shouldLog())
		{	
			Object[] args = new Object[] {  toCall.getLocation().getSourceIdentity(),mVerb,mClosure};
			String msg=aContext.formatMessage("MSG_LOG_CLOSURE",args);
			ILogger logger=aContext.getKernelContext().getKernel().getLogger();
			logger.logRaw(ILogger.DEBUG, DPMLRuntime.getInstance(),msg);
		}		
		
		try
		{	ctx.onCommissionSpace(aContext.getKernelContext().getKernel());
		} catch (Exception e)
		{ // won't happen
		}
		
		if (toCall instanceof IValueVariable)
		{	result =  ((IValueVariable)toCall).getValue(aContext,rctx,aRepresentationClass);
		}
		else
		{	throw aContext.createFormattedException("EX_DPML_RUNTIME","MSG_RUN_IDENTIFIER_NOT_IMPL",null,null,mClosure);
		}
		
		
		return result;			
	}

	public String toString()
	{	return "Closure["+mClosure+"]";
	}

}
