package org.netkernel.lang.dpml.ast.impl;

import org.netkernel.lang.dpml.ast.*;
import org.netkernel.lang.dpml.endpoint.DPMLRuntime;
import org.netkernel.layer0.nkf.INKFRequest;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.NKFException;
import org.netkernel.layer0.nkf.impl.NKFRequestImpl;
import org.netkernel.request.IRequestScopeLevel;
import org.netkernel.urii.IIdentifier;

import java.net.URI;
import java.net.URISyntaxException;


public class Dereference extends VariableImpl implements IIdentifierVariable
{
	private final IVariable mSource;
	
	public Dereference(String aName, IScope aScope, ILocation aLocation, IVariable aSource)
	{	super(aName,aScope,aLocation);
		mSource=aSource;
	}

	public String getIdentifier(INKFRequestContext aContext, IRequestScopeLevel aSubRequestContext) throws NKFException
	{
		IVariable var= DPMLRuntime.dereference(mSource,aContext,aSubRequestContext);
		IIdentifier rep;
		if (var instanceof IIdentifierVariable)
		{	String innerId= ((IIdentifierVariable)var).getIdentifier(aContext,aSubRequestContext);
			INKFRequest req=aContext.createRequest(innerId);
			((NKFRequestImpl)req).setRequestScope(aSubRequestContext);
			req.setRepresentationClass(IIdentifier.class);
			rep=(IIdentifier)aContext.issueRequest(req);
		}
		else
		{	IValue value = ((IValueVariable)var).getValue(aContext,aSubRequestContext,IIdentifier.class);
			rep = (IIdentifier) DPMLRuntime.ensureClassOfValue(value,IIdentifier.class,aContext);
		}
		String uriString=rep.toString();
		try
		{
			URI identifier=new URI(uriString);
			if (identifier!=null)
			{	return uriString;
			}
			return uriString;
		}
		catch (URISyntaxException e)
		{	throw aContext.createFormattedException("EX_DPML_RUNTIME","MSG_BAD_URI", DPMLRuntime.formatLocationMessage(var,aContext),null,uriString);
		}
	}
	
	public boolean isAbsolute()
	{	return false;
	}
}
