package org.netkernel.lang.dpml.ast.impl;

import org.netkernel.lang.dpml.ast.*;
import org.netkernel.lang.dpml.endpoint.CompilerScope;
import org.netkernel.layer0.nkf.INKFRequest;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFResponseReadOnly;
import org.netkernel.layer0.nkf.NKFException;
import org.netkernel.layer0.nkf.impl.NKFResponseReadOnlyImpl;
import org.netkernel.request.IRequestScopeLevel;
import org.netkernel.request.impl.RequestScopeLevelImpl;

import java.util.*;

public class Sequence extends VariableImpl implements IScope, IValueVariable
{
	private List<IVariable> mAssignments;
	private CompilerScope mCompilerScope;
	
	public Sequence(String aName, IScope aScope, ILocation aLocation, CompilerScope aCompilerScope)
	{	super(aName,aScope,aLocation);
		mAssignments=new ArrayList<IVariable>();
		mCompilerScope=aCompilerScope;
	}

	public void addAssignment(IVariable aVariable, boolean aReplace) throws NKFException
	{	mAssignments.add(aVariable);
	}
	
	public void addImport(IScope aImport, int aPrecedence)
	{
	}

	public void setResponse(IVariable response)
	{
	}
	
	public void close(INKFRequestContext aContext) throws NKFException
	{
	}	

	public IScope getParent()
	{	return getScope();
	}

	public Collection<IVariable> getVariables()
	{	return mAssignments;
	}

	public IVariable resolveVariable(String aName, IRequestScopeLevel aDynamicScope)
	{	IVariable result=null;
	
		IRequestScopeLevel scope=aDynamicScope;
		while (result==null && scope!=null)
		{	if (scope.getSpace() instanceof SequenceAssignmentsSpace)
			{	SequenceAssignmentsSpace sas=(SequenceAssignmentsSpace)scope.getSpace();
				if (sas.getOwner()==this)
				{	result=sas.getAssignment(aName);
					break;
				}
			}
			scope=scope.getParent();
		}
		return result;
	}

	public IValue getValue(INKFRequestContext aContext, IRequestScopeLevel aSubRequestContext, Class aRepresentationClass) throws NKFException
	{
		SequenceAssignmentsSpace previous=SequenceAssignmentsSpace.sRoot;
		IRequestScopeLevel scope = RequestScopeLevelImpl.createDurableScopeLevel(previous, aSubRequestContext);
		
		for (IVariable v : mAssignments)
		{	String name=v.getName();
			IVariable assignment=null;
			if (v instanceof IValueVariable)
			{	IValueVariable vv = (IValueVariable)v;
				IValue value=((IValueVariable) v).getValue(aContext, scope, Object.class);
				
				if (name!=null)
				{	assignment=new LiteralValue(name,null,getLocation(),value.getRepresentation(),value.getMeta());
				}
			}
			else
			{	assignment=v;
			}
			
			if (name!=null && assignment!=null)
			{	previous = new SequenceAssignmentsSpace(this, name, assignment, previous);
				scope=RequestScopeLevelImpl.createDurableScopeLevel(previous, aSubRequestContext);
			}
		}
		
		IVariable resp = null;
		if (previous!=SequenceAssignmentsSpace.sRoot)
		{	resp = previous.getAssignment(Closure.RESPONSE);
		}

		IValue result;
		if (resp instanceof IIdentifierVariable)
		{	
			IIdentifierVariable idVar = (IIdentifierVariable)resp;
			String id=idVar.getIdentifier(aContext,aSubRequestContext);
			INKFRequest req=aContext.createRequest(id);
			req.setVerb(aContext.getThisRequest().getVerb());
			req.setRequestScope(aSubRequestContext);
			req.setRepresentationClass(aRepresentationClass);
			INKFResponseReadOnly r=aContext.issueRequestForResponse(req);
			result = new ValueImpl(r.getRepresentation(),((NKFResponseReadOnlyImpl)r).getMeta());
		}
		else if (resp!=null)
		{	result= ((IValueVariable)resp).getValue(aContext,aSubRequestContext,aRepresentationClass);
		}
		else
		{	result=null;
		}
		
		return result;
	}

	public CompilerScope getCompilerScope()
	{	return mCompilerScope;
	}
}
