package org.netkernel.lang.dpml.endpoint;

import org.netkernel.lang.dpml.ast.IVariable;
import org.netkernel.lang.dpml.representation.DPMLRepresentation;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFRequestReadOnly;
import org.netkernel.layer0.nkf.INKFResolutionContext;
import org.netkernel.layer0.nkf.impl.NKFCombinedEndpointSpaceImpl;
import org.netkernel.request.IRequestScopeLevel;
import org.netkernel.urii.ClassLoaderWithExports;
import org.netkernel.urii.ISpace;

/** A Context that provides access to all the static closures, requests and literals in a DPML Script
 * @author Tony Butterfield
 */
public class DPMLScriptContext extends NKFCombinedEndpointSpaceImpl
{
	private DPMLRepresentation mScript;
	private ISpace mDPMLContext;
	private String mCWU;
	
	public static final String CLOSURE_SCHEME="dpml:";
	
	public DPMLScriptContext(DPMLRepresentation aScript, ISpace aDPMLContext, String aCWU)
	{	super(true);
		declareVerboseUnhandledExceptions(false);
		mScript=aScript;
		mDPMLContext=aDPMLContext;
		mCWU=aCWU;
	}

public String toString()
	{	return "DPMLScriptContext("+((IVariable)mScript.getRootScope()).getLocation().getSourceIdentity()+")";
	}
	
	////////////////////////////////
	// ISpace interface
	////////////////////////////////

	////////////////////////////////
	// NKFCombinedEndpointSpaceImpl interface
	////////////////////////////////
	public void onSource(INKFRequestContext aContext) throws Exception
	{	aContext.setCWU(mCWU);
		String id=aContext.getThisRequest().getIdentifier();
		int i=id.indexOf(':');
		id=id.substring(i+1);
		
		IVariable var = mScript.resolveId(id);
		IRequestScopeLevel rctx = aContext.getKernelContext().getThisKernelRequest().getRequestScope();
		DPMLRuntime.processRequest3(aContext,var,rctx,null);
	}
	
	public void onResolve(INKFResolutionContext aContext) throws Exception
	{
		INKFRequestReadOnly toResolve = aContext.getRequestToResolve();
		if (toResolve.getVerb()==INKFRequestReadOnly.VERB_SOURCE)
		{
			String identifier=toResolve.getIdentifier();
			// look for any referenced closures
			if (identifier.startsWith(CLOSURE_SCHEME) && mScript!=null )
			{	String name;
				int plusIndex=identifier.indexOf('+');
				if (plusIndex>=0)
				{	name = identifier.substring(CLOSURE_SCHEME.length(),plusIndex);
				}
				else
				{	name = identifier.substring(CLOSURE_SCHEME.length());
				}
				if (mScript.resolveId(name)!=null)
				{	aContext.createResolutionResponse(this);
				}
			}
		}
	}
}
