package org.netkernel.lang.dpml.endpoint;

import org.netkernel.layer0.nkf.INKFRequest;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFResponseReadOnly;
import org.netkernel.layer0.nkf.NKFException;
import org.netkernel.layer0.representation.WrappedThrowable;
import org.netkernel.layer0.urii.ValueSpace;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.netkernel.request.impl.MetaImpl;
import org.netkernel.util.Utils;

public class ExceptionAccessor extends StandardAccessorImpl
{
	public ExceptionAccessor()
	{	declareThreadSafe();
		declareUnhandledExceptionsExpired(false);
		declareVerboseUnhandledExceptions(false);
	}
	
	public void onSource(INKFRequestContext aContext) throws Exception
	{
		Class representationClass = aContext.getThisRequest().getRepresentationClass();
		try
		{	INKFResponseReadOnly tryBlockResponse = aContext.sourceForResponse("arg:try", representationClass);
			aContext.createResponseFrom(tryBlockResponse);
		}
		catch (NKFException e)
		{	if (aContext.getThisRequest().argumentExists("catch"))
			{	Throwable toUse=e;
				if (e.getId().equals(NKFException.class.getName()))
				{	toUse=e.getCause();
				}
				
				ValueSpace vs = new ValueSpace(2);
				System.out.println(Utils.throwableToString(toUse));
				vs.put("dpml-arg:exception", new WrappedThrowable(toUse), MetaImpl.NEVER_EXPIRED);
				vs.put("dpml-arg:exception-id", e.getDeepestId(), MetaImpl.NEVER_EXPIRED);
				vs.put("dpml-arg:exception-message", e.getDeepestMessage(), MetaImpl.NEVER_EXPIRED);
				INKFRequest req=aContext.createRequest("arg:catch");
				req.injectDurableRequestScope(vs);
				req.setRepresentationClass(representationClass);
				INKFResponseReadOnly catchBlockResponse = aContext.issueRequestForResponse(req);
				aContext.createResponseFrom(catchBlockResponse);
			}
			else
			{	throw e;
			}
		}
		finally
		{	if (aContext.getThisRequest().argumentExists("finally"))
			{	INKFResponseReadOnly finallyBlockResponse = aContext.sourceForResponse("arg:finally", representationClass);
				aContext.createResponseFrom(finallyBlockResponse);
			}
		}
	}
}
